/*
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2021 Timm Fitschen <t.fitschen@indiscale.com>
 * Copyright (C) 2021 IndiScale GmbH <info@indiscale.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 */

/**
 * @brief Creation and execution of transactions.
 */

#ifndef CAOSDB_TRANSACTION_H
#define CAOSDB_TRANSACTION_H

#include <memory>
#include "caosdb/entity.h"
#include "caosdb/entity/v1alpha1/main.pb.h"
#include "caosdb/entity/v1alpha1/main.grpc.pb.h"

namespace caosdb::transaction {
using caosdb::entity::Entity;
using caosdb::entity::EntityID;
using ProtoEntity = caosdb::entity::v1alpha1::Entity;
using caosdb::entity::v1alpha1::EntityTransactionService;
using caosdb::entity::v1alpha1::RetrieveRequest;

class ResultSet {
public:
  [[nodiscard]] auto GetById(EntityID id) const -> Entity;
};

class UniqueResult : public ResultSet {
private:
  Entity entity;

public:
  UniqueResult(ProtoEntity entity);
};

enum TransactionState { INIT = 10, EXECUTING = 20, SUCCESS = 30, ERROR = 40 };

/**
 * @brief Create a transaction via `CaosDBConnection.createTransaction()`
 */
class Transaction {
private:
  std::shared_ptr<ResultSet> result_set;
  TransactionState state = TransactionState::INIT;
  std::shared_ptr<EntityTransactionService::Stub> service_stub;
  RetrieveRequest request; // TODO(tf)

public:
  Transaction(std::shared_ptr<EntityTransactionService::Stub> service_stub);
  auto Retrieve(const EntityID &id) -> void;
  auto Execute() -> void;
  [[nodiscard]] auto GetResultSet() const -> std::shared_ptr<ResultSet>;
  auto WaitForIt() const -> void;
};

} // namespace caosdb::transaction
#endif
