/*
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2021 Timm Fitschen <t.fitschen@indiscale.com>
 * Copyright (C) 2021 IndiScale GmbH <info@indiscale.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "caosdb/entity.h"
#include "caosdb/exceptions.h" // for AuthenticationError
#include "caosdb/transaction.h"
#include <memory>
#include <utility>
#include <iostream>                               // for basic_ostream::ope...
#include <stdexcept>                              // for runtime_error
#include <grpcpp/impl/codegen/client_context.h>   // for ClientContext
#include <grpcpp/impl/codegen/status.h>           // for Status
#include "grpcpp/impl/codegen/status_code_enum.h" // for StatusCode, UNAUTH...
#include "caosdb/entity/v1alpha1/main.pb.h"
#include "caosdb/entity/v1alpha1/main.grpc.pb.h"

namespace caosdb::transaction {
using caosdb::entity::v1alpha1::EntityTransactionService;
using caosdb::entity::v1alpha1::RetrieveRequest;
using caosdb::entity::v1alpha1::RetrieveResponse;
using caosdb::exceptions::AuthenticationError;
using caosdb::exceptions::ConnectionError;

Transaction::Transaction(
  std::shared_ptr<EntityTransactionService::Stub> service_stub) {
  this->service_stub = std::move(service_stub);
}

auto Transaction::Retrieve(const EntityID &id) -> void {

  RetrieveRequest request;
  request.mutable_entity()->set_eid(id.ToString());

  this->request = request;
}

auto Transaction::Execute() -> void {
  // TODO(tf) throw error if not int INIT state
  this->state = TransactionState::EXECUTING;

  RetrieveResponse response;
  grpc::ClientContext context;
  const grpc::Status status =
    this->service_stub->Retrieve(&context, this->request, &response);

  if (!status.ok()) {
    switch (status.error_code()) {
    case grpc::StatusCode::UNAUTHENTICATED:
      throw AuthenticationError(status.error_message());
    case grpc::StatusCode::UNAVAILABLE:
      throw ConnectionError(status.error_message());
    default:
      std::cout << status.error_code() << "\n";
      throw std::runtime_error(status.error_message());
    }
  }
  // TODO(tf)
  this->result_set = std::make_shared<ResultSet>();
}

auto Transaction::WaitForIt() const -> void {
  // TODO(tf) throw error if still in INIT state
  // TODO(tf)
}

[[nodiscard]] auto Transaction::GetResultSet() const
  -> std::shared_ptr<ResultSet> {
  return this->result_set;
}

} // namespace caosdb::transaction
