/*
 *
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2021 Timm Fitschen <t.fitschen@indiscale.com>
 * Copyright (C) 2021 IndiScale GmbH <info@indiscale.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 */

// A simple caosdb client
#include <iostream>
#include <memory>
#include <string>
#include "caosdb/constants.h"
#include "caosdb/connection.h"
#include "caosdb/authentication.h"
#include "caosdb/utils.h"
#include "caosdb/info.h"

auto main() -> int {

  std::cout << "CaosDB (libcaosdb " << caosdb::LIBCAOSDB_VERSION_MINOR << "."
            << caosdb::LIBCAOSDB_VERSION_MINOR << "."
            << caosdb::LIBCAOSDB_VERSION_PATCH << ")" << std::endl;
  std::cout << "We don't miss the H of caos." << std::endl;

  const auto pem_file =
    caosdb::utils::get_env_var("CAOSDB_SERVER_CA_PEM", std::string());
  const auto host =
    caosdb::utils::get_env_var("CAOSDB_SERVER_HOST", "localhost");
  const auto port_str =
    caosdb::utils::get_env_var("CAOSDB_SERVER_PORT", "8443");
  const auto port = std::stoi(port_str);
  const auto user = caosdb::utils::get_env_var("CAOSDB_USER", "admin");
  const auto password = caosdb::utils::get_env_var("CAOSDB_PASSWORD", "caosdb");
  std::cout << "USER: " << user << "\n";
  std::cout << "PORT: " << port_str << "\n";
  std::cout << "HOST: " << host << "\n";
  std::cout << "CACERT: " << pem_file << "\n";

  auto auth =
    std::make_shared<caosdb::authentication::PlainPasswordAuthenticator>(
      user, password);
  auto cacert =
    std::make_shared<caosdb::connection::PemFileCACertProvider>(pem_file);
  auto config = std::make_shared<caosdb::connection::SslCaosDBConnectionConfig>(
    host, port, cacert, auth);
  caosdb::connection::CaosDBConnection connection(config);
  std::cout << std::endl << connection << std::endl;
  const auto &v_info = connection.GetVersionInfo();
  std::cout << "VersionInfo(" << v_info->GetMajor() << "." << v_info->GetMinor()
            << "." << v_info->GetPatch() << "-" << v_info->GetPreRelease()
            << "-" << v_info->GetBuild() << ")" << std::endl;
  return 0;
}
